<?php

namespace Kahanit\TableRateShipping\Controller\Adminhtml\Index;

// inheritance
use Magento\Backend\App\Action;
// injection
use Magento\Backend\App\Action\Context;
use Magento\Framework\Json\Encoder;
use Magento\Framework\Json\Decoder;
use Magento\Framework\Controller\Result\Raw;
use Magento\Framework\Data\Form\FormKey;
use Magento\Framework\FileSystem;
use Kahanit\TableRateShipping\Model\CarrierTableRateFactory;
use Kahanit\TableRateShipping\Model\CarrierTableRateMethodFactory;
use Kahanit\TableRateShipping\Model\CarrierTableRateProductFactory;
use Kahanit\TableRateShipping\Model\CarrierTableRateZoneFactory;
use Magento\Catalog\Model\ProductFactory;
use Magento\Store\Model\StoreManagerInterface;
// other
use Kahanit\TableRateShipping\Helper\TRSHelper;
use Kahanit\TableRateShipping\Helper\TRSUploadHandler;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Store\Model\Store;

class Ajax extends Action
{
    protected $jsonEncoder;

    protected $jsonDecoder;

    protected $rawResult;

    protected $formKey;

    protected $fileSystem;

    protected $carrierTableRateFactory;

    protected $carrierTableRateMethodFactory;

    protected $carrierTableRateProductFactory;

    protected $carrierTableRateZoneFactory;

    protected $productFactory;

    protected $storeManager;

    public function __construct(
        Context $context,
        Encoder $jsonEncoder,
        Decoder $jsonDecoder,
        Raw $rawResult,
        FormKey $formKey,
        FileSystem $fileSystem,
        CarrierTableRateFactory $carrierTableRateFactory,
        CarrierTableRateMethodFactory $carrierTableRateMethodFactory,
        CarrierTableRateProductFactory $carrierTableRateProductFactory,
        CarrierTableRateZoneFactory $carrierTableRateZoneFactory,
        ProductFactory $productFactory,
        StoreManagerInterface $storeManager
    )
    {
        $this->jsonEncoder = $jsonEncoder;
        $this->jsonDecoder = $jsonDecoder;
        $this->rawResult = $rawResult;
        $this->formKey = $formKey;
        $this->fileSystem = $fileSystem;
        $this->carrierTableRateFactory = $carrierTableRateFactory;
        $this->carrierTableRateMethodFactory = $carrierTableRateMethodFactory;
        $this->carrierTableRateProductFactory = $carrierTableRateProductFactory;
        $this->carrierTableRateZoneFactory = $carrierTableRateZoneFactory;
        $this->productFactory = $productFactory;
        $this->storeManager = $storeManager;

        parent::__construct($context);
    }

    public function execute()
    {
        $method = $this->getRequest()->getParam('method', false);
        $header = $this->getRequest()->getParam('header', 'json');

        if ($method !== false && method_exists($this, $method)) {
            if ($header == 'json') {
                $this->rawResult->setHeader('Content-Type', 'application/json');
                return $this->rawResult->setContents($this->jsonEncoder->encode($this->$method()));
            } elseif ($header == 'csv') {
                return $this->$method();
            } else {
                return $this->rawResult->setContents($this->$method());
            }
        }
        die();
    }

    /* functions group one */

    public function getRuleGroups()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $start = (int)$this->getRequest()->getParam('start', 0);
        $length = (int)$this->getRequest()->getParam('length', 20);
        $where = '';
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        $carrierTableRate = $this->carrierTableRateFactory->create();
        $records = $carrierTableRate->getResource()->getRuleGroups($id_carrier, $id_zone, $start, $length, $where, $id_shop);
        $total_records = $carrierTableRate->getResource()->getRuleGroupsNumRows($id_carrier, $id_zone, $where, $id_shop);

        return [
            'recordsTotal' => $total_records,
            'recordsFiltered' => $total_records,
            'data' => $records
        ];
    }

    public function getRules()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $id_country = $this->getRequest()->getParam('id_country', '0');
        $id_state = (int)$this->getRequest()->getParam('id_state', 0);
        $dest_city = $this->getRequest()->getParam('dest_city', '');
        $dest_zip = $this->getRequest()->getParam('dest_zip', '');
        $start = (int)$this->getRequest()->getParam('start', 0);
        $length = (int)$this->getRequest()->getParam('length', 20);
        $where = '';
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        $carrierTableRate = $this->carrierTableRateFactory->create();
        $records = $carrierTableRate->getResource()->getRules($id_carrier, $id_zone, $id_country, $id_state, $dest_city, $dest_zip, $start, $length, $where, $id_shop);
        $total_records = $carrierTableRate->getResource()->getRulesNumRows($id_carrier, $id_zone, $id_country, $id_state, $dest_city, $dest_zip, $where, $id_shop);

        return [
            'recordsTotal' => $total_records,
            'recordsFiltered' => $total_records,
            'data' => $records
        ];
    }

    /* functions group two */

    public function saveRules()
    {
        $action = $this->getRequest()->getParam('action', '');
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $records = $this->getRequest()->getParam('records', []);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->saveRules($action, $id_carrier, $records, $id_shop);
    }

    public function import()
    {
        $file = $this->getRequest()->getParam('file', '');
        $file = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR)->getAbsolutePath('import/Kahanit/TableRateShipping/') . $file;
        $csv_separator = $this->getRequest()->getParam('csv_separator', ',');
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->import($file, $csv_separator, $id_carrier, $id_shop);
    }

    public function export()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $csv_separator = $this->getRequest()->getParam('csv_separator', ',');
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        $this->carrierTableRateFactory->create()->getResource()->export($id_carrier, $csv_separator, $id_shop);
        die();
    }

    /* functions group three */

    public function updateCountryStateCity()
    {
        $update = $this->getRequest()->getParam('update', false);
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $id_country = $this->getRequest()->getParam('id_country', '0');
        $id_state = (int)$this->getRequest()->getParam('id_state', 0);
        $dest_city = $this->getRequest()->getParam('dest_city', false);
        $dest_zip = $this->getRequest()->getParam('dest_zip', '');
        $value = $this->getRequest()->getParam('value', false);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->updateCountryStateCity(
            $update, $id_carrier, $id_zone, $id_country, $id_state, $dest_city, $dest_zip, $value, $id_shop
        );
    }

    public function updateConditionPriceComment()
    {
        $update = $this->getRequest()->getParam('update', false);
        $id_carrier_table_rate = $this->getRequest()->getParam('id_carrier_table_rate', false);
        $value = $this->getRequest()->getParam('value', false);

        return $this->carrierTableRateFactory->create()->getResource()->updateConditionPriceComment(
            $update, $id_carrier_table_rate, $value
        );
    }

    public function updateStatus()
    {
        $id_carrier_table_rate = $this->getRequest()->getParam('id_carrier_table_rate', false);
        $active = (int)$this->getRequest()->getParam('active', 1);

        return $this->carrierTableRateFactory->create()->getResource()->updateStatus($id_carrier_table_rate, $active);
    }

    public function updateOrder()
    {
        $entity = $this->getRequest()->getParam('entity', '');
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $order = $this->getRequest()->getParam('order', []);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->updateOrder($entity, $id_carrier, $id_zone, $order, $id_shop);
    }

    public function updateProductSelected()
    {
        $id_product = (int)$this->getRequest()->getParam('id_product', 0);
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $select = (int)$this->getRequest()->getParam('select', 1);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateProductFactory->create()->getResource()->updateProductSelected($id_product, $id_carrier, $select, $id_shop);
    }

    /* functions group four */

    public function deleteRules()
    {
        $entity = $this->getRequest()->getParam('entity', '');
        $type = $this->getRequest()->getParam('type', '');
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $data = $this->getRequest()->getParam('data', false);
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $id_carrier_table_rate = $this->getRequest()->getParam('id_carrier_table_rate', false);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->deleteRules(
            $entity, $type, $id_carrier, $data, $id_zone, $id_carrier_table_rate, $id_shop
        );
    }

    /* functions group four */

    public function processAttachment()
    {
        $dir = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR)->getAbsolutePath('import/Kahanit/TableRateShipping/');
        $url = $this->getUrl('tablerateshipping/index/ajax', ['store' => 1, 'form_key' => $this->formKey->getFormKey()]) . '?isAjax=true';

        $uploads = new TRSUploadHandler(
            [
                'accept_file_types' => '/(\.|\/)(csv)$/i',
                'upload_dir' => $dir,
                'upload_url' => $url . '&method=downloadAttachment&header=csv&file=',
                'script_url' => $url . '&method=processAttachment',
                'print_response' => false
            ]
        );

        return $uploads->get_response();
    }

    public function downloadAttachment()
    {
        $file = $this->getRequest()->getParam('file', '');

        if ($file == '') {
            return;
        }

        $file = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR)->getAbsolutePath('import/Kahanit/TableRateShipping/') . $file;

        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename=' . basename($file));
        header('Expires: 0');
        header('Cache-Control: must-revalidate');
        header('Pragma: public');
        header('Content-Length: ' . filesize($file));

        readfile($file);
        exit;
    }

    /* functions group six */

    public function getProducts()
    {
        /* init */
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);
        $product = $this->productFactory->create();
        $collection = $product->getCollection()
            ->addFieldToSelect('name')
            ->addFieldToSelect('sku')
            ->addFieldToSelect('status')
            ->joinTable(
                ['ctp' => $product->getResource()->getTable('carrier_table_rate_product')],
                'id_product = entity_id',
                ['id_product', 'id_carrier'],
                ['id_carrier' => $id_carrier, 'id_shop' => $id_shop],
                'left'
            )
            ->addStoreFilter($id_shop);

        /* search */
        $search = $this->getRequest()->getParam('search', ['value' => '']);
        $search_filtered = TRSHelper::filterSearchQuery(
            $search['value'], ['id', 'name']
        );
        if ($search_filtered['id'] != '') {
            $collection->addFieldToFilter(
                'entity_id', ['=' => $search_filtered['id']]
            );
        }
        if ($search_filtered['name'] != '') {
            $collection->addFieldToFilter(
                'name', ['like' => '%' . $search_filtered['name'] . '%']
            );
        }
        if ($search_filtered['id'] == '' && $search_filtered['name'] == ''
            && $search_filtered['query'] != ''
        ) {
            $collection->addFieldToFilter(
                'name', ['like' => '%' . $search_filtered['query'] . '%']
            );
        }

        /* order */
        $order = $this->getRequest()->getParam('order', [['column' => 0, 'dir' => 'ASC']]);
        $columns = $this->getRequest()->getParam('columns', []);
        if (isset($columns[$order[0]['column']]['data'])) {
            $orderfld = strtolower($columns[$order[0]['column']]['data']);
            if ($orderfld === 'text') {
                $orderfld = 'name';
            } else {
                $orderfld = 'entity_id';
            }
        } else {
            $orderfld = '';
        }
        if (isset($order[0]['dir'])) {
            $orderdir = strtoupper($order[0]['dir']);
        } else {
            $orderdir = '';
        }
        if ($orderfld !== '' && $orderdir !== '') {
            $collection->addOrder($orderfld, $orderdir);
        }

        /* page */
        $start = (int)$this->getRequest()->getParam('start', 0);
        $length = (int)$this->getRequest()->getParam('length', 25);
        $page = $start / $length + 1;
        $collection->setPageSize($length);
        $collection->setCurPage($page);

        /* prepage */
        $products = [];
        $c = 0;
        foreach ($collection->getItems() as $item) {
            $products[$c] = [
                'id_product' => $item->getData('entity_id'),
                'name' => $item->getData('name'),
                'reference' => $item->getData('sku'),
                'id_category_default' => '',
                'name_category' => '',
                'active' => ($item->getData('status') == 2) ? 0 : $item->getData('status'),
                'hascarrier' => $item->getData('id_carrier') != ''
            ];

            $c++;
        }

        /* return */
        return [
            'recordsTotal' => $collection->getSize(),
            'recordsFiltered' => $collection->getSize(),
            'data' => $products
        ];
    }

    public function getImportStatus()
    {
        $dir_var = $this->fileSystem->getDirectoryWrite(DirectoryList::VAR_DIR);
        if (!$dir_var->isFile('import/Kahanit/TableRateShipping/ImportStatus.json')) {
            $dir_var->writeFile('import/Kahanit/TableRateShipping/ImportStatus.json', '{"rowImported":0}');
        }

        $import_status = $this->jsonDecoder->decode($dir_var->readFile('import/Kahanit/TableRateShipping/ImportStatus.json'));
        if (is_array($import_status) && isset($import_status['rowImported'])) {
            return $import_status;
        } else {
            return ['rowImported' => 'Wait..'];
        }
    }

    public function getZonesOrder()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->getZonesOrder($id_carrier, $id_shop);
    }

    public function getCountries()
    {
        $search = $this->getRequest()->getParam('q', '');
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->getCountries($search, $id_shop);
    }

    public function getStates()
    {
        $search = $this->getRequest()->getParam('q', '');
        $id_country = $this->getRequest()->getParam('c', '');
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);

        return $this->carrierTableRateFactory->create()->getResource()->getStates($search, $id_country, $id_shop);
    }

    /* functions group seven */

    public function saveCarrier()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);
        $carrier_name = TRSHelper::addslashes($this->getRequest()->getParam('carrier_name', ''));
        $method_name = TRSHelper::addslashes($this->getRequest()->getParam('method_name', ''));

        if ($id_carrier == 0) {
            $carrier = $this->carrierTableRateMethodFactory->create();
        } else {
            $carrier = $this->carrierTableRateMethodFactory->create()->load($id_carrier);
            if ($carrier->isEmpty()) {
                return [
                    'status' => 'danger',
                    'message' => 'Carrier not found'
                ];
            }
        }

        $carrier->setIdShop($id_shop);
        $carrier->setCarrierName($carrier_name);
        $carrier->setMethodName($method_name);
        $message = $carrier->getResource()->validateFields($carrier);

        if ($message == 1) {
            $carrier->save();

            return [
                'status' => 'success',
                'message' => 'Carrier saved successfully',
                'id_carrier' => $carrier->getId(),
                'carrier_name' => TRSHelper::stripslashes($carrier->getCarrierName()),
                'method_name' => TRSHelper::stripslashes($carrier->getMethodName())
            ];
        } else {
            return [
                'status' => 'danger',
                'message' => $message
            ];
        }
    }

    public function deleteCarrier()
    {
        $id_carrier = (int)$this->getRequest()->getParam('id_carrier', 0);

        if ($id_carrier != 0) {
            $carrier = $this->carrierTableRateMethodFactory->create()->load($id_carrier);
            if ($carrier->isEmpty()) {
                return [
                    'status' => 'danger',
                    'message' => 'Carrier not found'
                ];
            }

            $carrier->delete();
            $this->carrierTableRateFactory->create()->getResource()->deleteRulesByCarrier($id_carrier);
            $this->carrierTableRateProductFactory->create()->getResource()->deleteByCarrier($id_carrier);

            return [
                'status' => 'success',
                'message' => 'Carrier deleted successfully'
            ];
        } else {
            return [
                'status' => 'danger',
                'message' => 'Invalid carrier'
            ];
        }
    }

    /* functions group eight */

    public function saveZone()
    {
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);
        $id_shop = (int)$this->getRequest()->getParam('store', Store::DEFAULT_STORE_ID);
        $zone_name = TRSHelper::addslashes($this->getRequest()->getParam('zone_name', ''));

        if ($id_zone == 0) {
            $zone = $this->carrierTableRateZoneFactory->create();
        } else {
            $zone = $this->carrierTableRateZoneFactory->create()->load($id_zone);
            if ($zone->isEmpty()) {
                return [
                    'status' => 'danger',
                    'message' => 'Zone not found'
                ];
            }
        }

        $zone->setIdShop($id_shop);
        $zone->setName($zone_name);
        $message = $zone->getResource()->validateFields($zone);

        if ($message == 1) {
            $zone->save();

            return [
                'status' => 'success',
                'message' => 'Zone saved successfully',
                'id_zone' => $zone->getId(),
                'zone_name' => TRSHelper::stripslashes($zone->getName())
            ];
        } else {
            return [
                'status' => 'danger',
                'message' => $message
            ];
        }
    }

    public function deleteZone()
    {
        $id_zone = (int)$this->getRequest()->getParam('id_zone', -1);

        if ($id_zone != 0) {
            $zone = $this->carrierTableRateZoneFactory->create()->load($id_zone);
            if ($zone->isEmpty()) {
                return [
                    'status' => 'danger',
                    'message' => 'Zone not found'
                ];
            }

            $zone->delete();
            $this->carrierTableRateFactory->create()->getResource()->deleteRulesByZone($id_zone);

            return [
                'status' => 'success',
                'message' => 'Zone deleted successfully'
            ];
        } else {
            return [
                'status' => 'danger',
                'message' => 'Invalid zone'
            ];
        }
    }
}
