<?php

namespace Kahanit\TableRateShipping\Helper;

use Symfony\Component\ExpressionLanguage\ExpressionFunction;
use Symfony\Component\ExpressionLanguage\ExpressionFunctionProviderInterface;

class TRSExpressionFunctionProvider implements ExpressionFunctionProviderInterface
{
    public function getFunctions()
    {
        return [
            new ExpressionFunction('sin', function ($num) {
                return sprintf('(is_numeric(%1$s) ? sin(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return sin($num);
            }),
            new ExpressionFunction('cos', function ($num) {
                return sprintf('(is_numeric(%1$s) ? cos(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return cos($num);
            }),
            new ExpressionFunction('tan', function ($num) {
                return sprintf('(is_numeric(%1$s) ? tan(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return tan($num);
            }),
            new ExpressionFunction('csc', function ($num) {
                return sprintf('(is_numeric(%1$s) ? csc(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                $tmp = sin($num);
                if ($tmp == 0) {
                    throw new \Exception("Division by 0 on: 'csc({$num}) = 1/sin({$num})'", 5501);
                }

                return 1 / $tmp;
            }),
            new ExpressionFunction('sec', function ($num) {
                return sprintf('(is_numeric(%1$s) ? sec(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                $tmp = cos($num);
                if ($tmp == 0) {
                    throw new \Exception("Division by 0 on: 'sec({$num}) = 1/cos({$num})'", 5501);
                }

                return 1 / $tmp;
            }),
            new ExpressionFunction('cot', function ($num) {
                return sprintf('(is_numeric(%1$s) ? cot(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                $tmp = tan($num);
                if ($tmp == 0) {
                    throw new \Exception("Division by 0 on: 'cot({$num}) = 1/tan({$num})'", 5501);
                }

                return 1 / $tmp;
            }),
            new ExpressionFunction('abs', function ($num) {
                return sprintf('(is_numeric(%1$s) ? abs(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return abs($num);
            }),
            new ExpressionFunction('log', function ($num) {
                return sprintf('(is_numeric(%1$s) ? log(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                $ans = log($num);
                if (is_nan($ans) || is_infinite($ans)) {
                    throw new \Exception("Result of 'log({$num}) = {$ans}' is either infinite or a non-number", 5504);
                }

                return $ans;
            }),
            new ExpressionFunction('log10', function ($num) {
                return sprintf('(is_numeric(%1$s) ? log10(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                $ans = log10($num);
                if (is_nan($ans) || is_infinite($ans)) {
                    throw new \Exception("Result of 'log10({$num}) = {$ans}' is either infinite or a non-number", 5504);
                }

                return $ans;
            }),
            new ExpressionFunction('sqrt', function ($num) {
                return sprintf('(is_numeric(%1$s) ? sqrt(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return sqrt($num);
            }),
            new ExpressionFunction('ceil', function ($num) {
                return sprintf('(is_numeric(%1$s) ? ceil(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return ceil($num);
            }),
            new ExpressionFunction('floor', function ($num) {
                return sprintf('(is_numeric(%1$s) ? floor(%1$s) : %1$s)', $num);
            }, function ($arguments, $num) {
                if (!is_numeric($num)) {
                    return $num;
                }

                return floor($num);
            })
        ];
    }
}