<?php

namespace Kahanit\TableRateShipping\Model\ResourceModel;

// inheritance
use Magento\Framework\Model\ResourceModel\Db\AbstractDb;
// injection
use Magento\Framework\Model\ResourceModel\Db\Context;
// other
use Kahanit\TableRateShipping\Helper\TRSHelper;

class CarrierTableRateMethod extends AbstractDb
{
    public function __construct(
        Context $context,
        $connectionName = null
    )
    {
        parent::__construct($context, $connectionName);
    }

    protected function _construct()
    {
        $this->_init('carrier_table_rate_method', 'id_carrier');
    }

    public function validateFields(\Kahanit\TableRateShipping\Model\CarrierTableRateMethod $carrierTableRateMethod)
    {
        // validate carrier name
        if ($carrierTableRateMethod->getCarrierName() == ''
            || !preg_match('/^[^!<>;?=+@#"°{}_$%]*$/u', $carrierTableRateMethod->getCarrierName())
        ) {
            return 'Invalid `Carrier name`';
        }

        // validate method name
        if ($carrierTableRateMethod->getMethodName() == ''
            || !preg_match('/^[^!<>;?=+@#"°{}_$%]*$/u', $carrierTableRateMethod->getMethodName())
        ) {
            return 'Invalid `Method name`';
        }

        return (int)true;
    }

    public function getCarriers($id_shop, $include_default = false)
    {
        $connection = $this->getConnection();
        $sql = $connection->select();
        $sql->from($this->getMainTable(), ['*']);
        $sql->where('id_shop = ' . (int)$id_shop . (($include_default) ? ' OR id_shop = 0' : ''));

        $carriers = $connection->fetchAll($sql);
        foreach ($carriers as &$carrier) {
            $carrier['carrier_name'] = TRSHelper::stripslashes($carrier['carrier_name']);
            $carrier['method_name'] = TRSHelper::stripslashes($carrier['method_name']);
        }

        return $carriers;
    }
}
